/* Lehasoft Tetris Mobile 2006
   Copyright (c) Alexey Inkin, Lehasoft (1996-2006), GetSoft.ru, leha@inkin.ru

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  */

package Game;

import Main.*;

public class ScoresTable
{
    private SettingsKeeper      m_objSettingsKeeper;
    private ScoresTableRecord   m_objTableRecords[];
    private int                 m_nRecordsCount;
    
    public static  final int       RECORDS_COUNT       = 30;
    private static final String    PLAYER_NAME_PARAM   = "PlayerName";
    private static final String    PLAYER_SCORES_PARAM = "PlayerScores";
    
    public ScoresTable(SettingsKeeper sk)
    {
        m_objSettingsKeeper = sk;
        m_objTableRecords = m_objSettingsKeeper.loadScoresTable();
        m_nRecordsCount = m_objSettingsKeeper.getScoresTableRecordCount();
    }
    
    public void addTableRecord(String   strName,
                               int      nScores)
    {
        int i;
        int nScoresPosition = getScoresPosition(nScores);
        
        if (nScoresPosition > 0)
        {
            nScoresPosition--;
            for (i = RECORDS_COUNT - 1; i > nScoresPosition; i--)
                m_objTableRecords[i] = m_objTableRecords[i - 1];
            m_objTableRecords[nScoresPosition] = new ScoresTableRecord(strName, nScores);
            if (m_nRecordsCount < RECORDS_COUNT)
                m_nRecordsCount++;
            m_objSettingsKeeper.saveScoresTable(m_objTableRecords);
        }
    }
    
    public void cleanTable()
    {
        int i;
        
        m_objSettingsKeeper.clearScoresTable();
        m_objTableRecords = new ScoresTableRecord[RECORDS_COUNT];
        m_nRecordsCount = 0;
    }
    
    public int getLeaderScores()
    {
        if (m_objTableRecords[0] != null)
            return m_objTableRecords[0].getScores();
        return 0;
    }
    
    public String getNameAtPosition(int nPosition)
    {
        if (nPosition > 0 && nPosition <= m_nRecordsCount)
            return m_objTableRecords[nPosition - 1].getName();
        return null;
    }
    
    public int getNextRecordScores(int nScores)
    {
        int i;
        int nScoresPosition = getScoresPosition(nScores);
        
        if (nScoresPosition == 1)
            return 0;
        if (nScores == 0)
        {
            for (i = 0; i < RECORDS_COUNT; i++)
                if (m_objTableRecords[i] == null)
                    break;
            return i == 0 ? 0: m_objTableRecords[i - 1].getScores();
        }
        if (nScoresPosition == 0)
            return m_objTableRecords[RECORDS_COUNT - 1].getScores();
        return m_objTableRecords[nScoresPosition - 2].getScores();
    }
    
    public int getPreviousRecordScores(int nScores)
    {
        int nScoresPosition = getScoresPosition(nScores);
        
        if (nScoresPosition == 0)
            return 0;
        if (m_objTableRecords[nScoresPosition - 1] != null)
            return m_objTableRecords[nScoresPosition - 1].getScores();
        return 0;
    }

    public int getRecordsCount()
    {
        return m_nRecordsCount;
    }
    
    public int getScoresAtPosition(int nPosition)
    {
        if (nPosition > 0 && nPosition <= m_nRecordsCount)
            return m_objTableRecords[nPosition - 1].getScores();
        return 0;
    }
    
    public int getScoresPosition(long nScores)
    {
        int i;
        
        if (nScores == 0)
            return 0;
        for (i = 0; i < RECORDS_COUNT && m_objTableRecords[i] != null; i++)
            if (m_objTableRecords[i].getScores() < nScores)
                break;
        if (i != RECORDS_COUNT)
            return i + 1;
        return 0;
    }
}