/* Lehasoft Tetris Mobile 2006
   Copyright (c) Alexey Inkin, Lehasoft (1996-2006), GetSoft.ru, leha@inkin.ru

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  */

package Interface;

public class Color
{
    private Color()
    {
    }

    public static int blue(int nColor)
    {
        return nColor & 0x0000FF;
    }
    
    public static int fromHSV(int   nHue,
                              int   nSaturation,
                              int   nValue)
    {
        int nRed;
        int nGreen;
        int nBlue;
        int nColorCircleSegment;
        int nMaxRGB;
        int nMidRGB;
        int nMinRGB;
        
        if (nSaturation == 0)
        {
            nRed    = nValue * 255 / 100;
            nGreen  = nRed;
            nBlue   = nRed;
        }
        else
        {
            nColorCircleSegment = nHue / 60;
            nMaxRGB = 255 * nValue / 100;
            nMinRGB = 255 * nValue * (100 - nSaturation) / 100 / 100;
            nMidRGB = (nMaxRGB - nMinRGB) * (nHue % 60) / 60;
            if (nColorCircleSegment % 2 == 0)
                nMidRGB += nMinRGB;
            else
                nMidRGB = nMaxRGB - nMidRGB;
            switch (nColorCircleSegment)
            {
            case 0:
                nRed    = nMaxRGB;
                nGreen  = nMidRGB;
                nBlue   = nMinRGB;
		break;
            case 1:
                nRed    = nMidRGB;
                nGreen  = nMaxRGB;
                nBlue   = nMinRGB;
                break;
            case 2:
                nRed    = nMinRGB;
                nGreen  = nMaxRGB;
                nBlue   = nMidRGB;
                break;
            case 3:
                nRed    = nMinRGB;
                nGreen  = nMidRGB;
                nBlue   = nMaxRGB;
                break;
            case 4:
                nRed    = nMidRGB;
                nGreen  = nMinRGB;
                nBlue   = nMaxRGB;
                break;
            default:
                nRed    = nMaxRGB;
                nGreen  = nMinRGB;
                nBlue   = nMidRGB;
            }
        }
        return (nRed << 16) + (nGreen << 8) + nBlue;
    }
    
    public static int fromRGB(int   nRed,
                              int   nGreen,
                              int   nBlue)
    {
        return (nRed << 16) + (nGreen << 8) + nBlue;
    }
    
    public static int green(int nColor)
    {
        return (nColor & 0x00FF00) >> 8;
    }
    
    public static int hue(int nColor)
    {
        int nRed;
        int nGreen;
        int nBlue;
        int nMaxRGB;
        int nResult;
        
        nRed    = red(nColor);
        nGreen  = green(nColor);
        nBlue   = blue(nColor);
        nMaxRGB = Math.max(Math.max(nRed, nGreen), nBlue);
        
        if (nMaxRGB == 0)
            return 0;
        
        if (nMaxRGB == nRed)
            nResult = (nGreen - nBlue) * 60 / 255;
        else if (nMaxRGB == nGreen)
            nResult = 120 + (nBlue - nRed) * 60 / 255;
        else
            nResult = 240 + (nRed - nGreen) * 60 / 255;
        
        if (nResult < 0)
            return nResult + 360;
        if (nResult >= 360)
            return nResult - 360;
        return nResult;
    }

    public static int red(int nColor)
    {
        return (nColor & 0xFF0000) >> 16;
    }
    
    public static int saturation(int  nColor)
    {
        int nMaxRGB;
        int nMinRGB;
        int nRed;
        int nGreen;
        int nBlue;
        
        nRed    = red(nColor);
        nGreen  = green(nColor);
        nBlue   = blue(nColor);
        nMaxRGB = Math.max(Math.max(nRed, nGreen), nBlue);
        if (nMaxRGB == 0)
            return 0;
        nMinRGB = Math.min(Math.min(nRed, nGreen), nBlue);
        return (nMaxRGB - nMinRGB) * 100 / nMaxRGB;
    }
    
    public static int value(int    nColor)
    {
        return Math.max(Math.max(red(nColor), green(nColor)), blue(nColor)) * 100 / 255;
    }
}