/* Lehasoft Tetris Mobile 2006
   Copyright (c) Alexey Inkin, Lehasoft (1996-2006), GetSoft.ru, leha@inkin.ru

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  */

package Interface;

import Main.*;
import javax.microedition.lcdui.*;

public class LanguageMenu extends List implements CommandListener, Localizable
{
    private int                 m_nLocaleIndex;
    private Tetris              m_objApplication;
    private Localizer           m_objLocalizer;
    private Command             m_cmdCancel;
    private LanguageInformation m_arrLanguageInformation[];
    
    public LanguageMenu(Tetris objApplication, Localizer objLocalizer)
    {
        super("", IMPLICIT);
        
        m_objApplication = objApplication;
        m_objLocalizer = objLocalizer;
        
        setCommandListener(this);
        addCommand(SELECT_COMMAND);
        localize();
        fillList();
    }
    
    public void commandAction(Command cmd, Displayable d)
    {
        if (cmd == SELECT_COMMAND)
            onOK();
        else
            m_objApplication.onBack();
    }
    
    private void fillList()
    {
        int i;
        int nLocale         = m_objLocalizer.getLocale();
        
        m_arrLanguageInformation = m_objLocalizer.getLanguageList();
        for (i = 0; i < m_arrLanguageInformation.length; i++)
        {
            append(m_arrLanguageInformation[i].getName(), null);
            if (m_arrLanguageInformation[i].getLocale() == nLocale)
                m_nLocaleIndex = i;
        }
    }
    
    public void localize()
    {
        if (m_cmdCancel != null)
            removeCommand(m_cmdCancel);
        m_cmdCancel = new Command(m_objLocalizer.get(Localizer.S_CANCEL), Command.CANCEL, 1);
        addCommand(m_cmdCancel);
        
        setTitle(m_objLocalizer.get(Localizer.S_LANGUAGE));
    }
    
    private void onOK()
    {
        int nLanguageIndex = getSelectedIndex();
        
        m_objApplication.setLocale(m_arrLanguageInformation[nLanguageIndex].getLocale());
        m_objApplication.onBack();
    }
    
    public void preShow()
    {
        setSelectedIndex(m_nLocaleIndex, true);
    }
}