/* Lehasoft Tetris Mobile 2006
   Copyright (c) Alexey Inkin, Lehasoft (1996-2006), GetSoft.ru, leha@inkin.ru

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  */

package Interface;

import Main.*;
import javax.microedition.lcdui.*;

public class MainMenu extends List implements CommandListener, Localizable
{
    private Tetris      m_objApplication;
    private Localizer   m_objLocalizer;
    private int         m_nState;

    private Command     m_cmdExit;

    private static final int  STATE_GAME_NOTSTARTED = 0;
    private static final int  STATE_GAME_PAUSED     = 3;
    
    public MainMenu(Tetris      objApplication,
                    Localizer   objLocalizer)
    {
        super("Lehasoft Tetris", List.IMPLICIT);
        m_objApplication = objApplication;
        m_objLocalizer = objLocalizer;

        addCommand(List.SELECT_COMMAND);
        setCommandListener(this);
        localize();
    }
    
    public void commandAction(Command       cmd,
                              Displayable   d)
    {
        if (cmd == m_cmdExit)
            m_objApplication.exit();
        else
        {
            switch (m_nState)
            {
            case STATE_GAME_NOTSTARTED:
                switch (getSelectedIndex())
                {
                case 0:
                    m_objApplication.startGame();
                    break;
                case 1:
                    m_objApplication.onPreferences(this);
                    break;
                case 2:
                    m_objApplication.onScoresTable(this);
                    break;
                case 3:
                    m_objApplication.onControlKeys(this);
                    break;
                case 4:
                    m_objApplication.onAbout(this);
                }
                break;
            case STATE_GAME_PAUSED:
                switch (getSelectedIndex())
                {
                case 0:
                    m_objApplication.resumeGame();
                    break;
                case 1:
                    m_objApplication.restartGame();
                    break;
                case 2:
                    m_objApplication.onPreferences(this);
                    break;
                case 3:
                    m_objApplication.onScoresTable(this);
                    break;
                case 4:
                    m_objApplication.onControlKeys(this);
                    break;
                case 5:
                    m_objApplication.onAbout(this);
                }
            }
        }
    }
    
    public void localize()
    {
        switch (m_nState)
        {
            case STATE_GAME_NOTSTARTED:
                showGameNotStarted();
                break;
            case STATE_GAME_PAUSED:
                showGamePaused();
        }
        if (m_cmdExit != null)
            removeCommand(m_cmdExit);
        m_cmdExit = new Command(m_objLocalizer.get(Localizer.S_EXIT), Command.EXIT, 0);
        addCommand(m_cmdExit);
    }
    
    public void showGameNotStarted()
    {
        deleteAll();
        
        append(m_objLocalizer.get(Localizer.S_START), null);
        append(m_objLocalizer.get(Localizer.S_PREFERENCES), null);
        append(m_objLocalizer.get(Localizer.S_HIGH_SCORES), null);
        append(m_objLocalizer.get(Localizer.S_CONTROL_KEYS), null);
        append(m_objLocalizer.get(Localizer.S_ABOUT), null);
        
        m_nState = STATE_GAME_NOTSTARTED;
    }
    
    public void showGamePaused()
    {
        deleteAll();
        
        append(m_objLocalizer.get(Localizer.S_RESUME), null);
        append(m_objLocalizer.get(Localizer.S_RESTART), null);
        append(m_objLocalizer.get(Localizer.S_PREFERENCES), null);
        append(m_objLocalizer.get(Localizer.S_HIGH_SCORES), null);
        append(m_objLocalizer.get(Localizer.S_CONTROL_KEYS), null);
        append(m_objLocalizer.get(Localizer.S_ABOUT), null);
        
        m_nState = STATE_GAME_PAUSED;
    }
}