/* Lehasoft Tetris Mobile 2006
   Copyright (c) Alexey Inkin, Lehasoft (1996-2006), GetSoft.ru, leha@inkin.ru

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  */

package Interface;

import Main.*;
import Game.*;
import java.lang.*;
import java.util.*;
import java.io.*;
import javax.microedition.lcdui.*;
import javax.microedition.media.*;

public class TetrisCanvas extends Canvas implements CommandListener, Localizable
{
    private int     m_nWidth;
    private int     m_nHeight;
    private int     m_nBrickSize;
    private int     m_nBrickDepth;
    private boolean m_bDrawPosition;
    
    private int     m_nAreaStartX;
    private int     m_nAreaStartY;
    private int     m_nAreaWidth;
    private int     m_nAreaHeight;
    private int     m_nPreviewStartX;
    private int     m_nPreviewSize;
    private int     m_nPositionCenterX;
    private int     m_nPositionStartY;
    private int     m_nPositionRadius;
    private int     m_nGaugeStartX;
    private int     m_nGaugeWidth;
    private int     m_nGaugeHeight;
    private int     m_nTotalGaugeStartY;
    private int     m_nNextPositionGaugeStartY;
    private int     m_nBannerStartX;
    private int     m_nBannerStartY;
    
    private String      m_strSideMessage                = "";
    private Random      m_objRandom                     = new Random();
    private Command     m_cmdMenu;
    private TetrisGame  m_objGame;
    private Tetris      m_objApplication;
    private ColorThemeKeeper    m_objColorThemeKeeper;
    private SideMessageTicker   m_objSideMessageTicker  = new SideMessageTicker(this);
    private Localizer   m_objLocalizer;
    private Image       m_imgLehasoftBanner;
    private Image       m_imgGetSoftBanner;
    private Image       m_imgCurrentBanner;
    private Image       m_imgBackground;
    private Image       m_imgBrick00;
    private Image       m_imgBrick10;
    private Image       m_imgBrick01;
    private Image       m_imgBrick11;
    private Image       m_imgBrick02;
    private Image       m_imgBrick12;
    private Font        m_objBigFont;
    private Font        m_objNormalFont;
    private Font        m_objSmallFont;
    private int         m_nBigFontLineHeight;
    private int         m_nNormalFontLineHeight;
    private int         m_nSmallFontLineHeight;
    
    private int         m_nBackgroundColor;
    private int         m_nBorderColor;
    private int         m_nBrickFaceColor;
    private int         m_nBrickTopColor;
    private int         m_nBrickRightColor;
    private int         m_nStatisticsColor;
    private int         m_nPauseMessageColor;
    private int         m_nGameOverMessageColor;
    private int         m_nGaugeColor;
    private int         m_nSideMessageColor;
    
    
    public TetrisCanvas(Tetris objApplication, Localizer objLocalizer)
    {
        int         nBannerWidth    = 0;
        int         nBrickWidth;
        int         nBrickHeight;
        Graphics    g;
        
        setTitle("Lehasoft Tetris");
        m_objApplication = objApplication;
        m_objLocalizer = objLocalizer;
        m_nWidth    = getWidth();
        m_nHeight   = getHeight();
        
        if (m_nHeight >= 96)
        {
            MostFitImage    imgLehasoftBanners = new MostFitImage();
            MostFitImage    imgGetSoftBanners = new MostFitImage();
            
            imgLehasoftBanners.addImage(13, 96, "/Images/lehasoft96.png");
            imgLehasoftBanners.addImage(17, 128, "/Images/lehasoft128.png");
            imgLehasoftBanners.addImage(21, 160, "/Images/lehasoft160.png");
            
            imgGetSoftBanners.addImage(13, 96, "/Images/getsoft96.png");
            imgGetSoftBanners.addImage(17, 128, "/Images/getsoft128.png");
            imgGetSoftBanners.addImage(21, 160, "/Images/getsoft160.png");
            
            imgLehasoftBanners.loadMostFitImage(21, m_nHeight);
            imgGetSoftBanners.loadMostFitImage(21, m_nHeight);
            
            m_imgLehasoftBanner = imgLehasoftBanners.getImage();
            m_imgGetSoftBanner = imgGetSoftBanners.getImage();
            
            nBannerWidth = m_imgLehasoftBanner.getWidth() + 2;
            m_nBannerStartX = 2;
            m_nBannerStartY = (m_nHeight - m_imgLehasoftBanner.getHeight()) / 2;
        }

        nBrickWidth = (m_nWidth - nBannerWidth - 2 - 6 - 2) * 10 / (TetrisGame.WIDTH * 10 + 50);
        nBrickHeight = (m_nHeight - 2 - 2) * 10 / (TetrisGame.HEIGHT * 10 + 5);
        m_nBrickSize = Math.min(nBrickWidth, nBrickHeight);
        m_nBrickDepth = m_nBrickSize / 3;
        
        m_nPreviewSize = m_nBrickSize * 45 / 10;
        m_nPreviewStartX = m_nWidth - m_nPreviewSize - 4;

        m_nAreaWidth = m_nBrickSize * (TetrisGame.WIDTH * 10 + 5) / 10;
        m_nAreaHeight = m_nBrickSize * (TetrisGame.HEIGHT * 10 + 5) / 10;
        m_nAreaStartY = (m_nHeight - m_nAreaHeight) / 2;
        m_nAreaStartX = nBannerWidth + (m_nPreviewStartX - m_nAreaWidth - nBannerWidth) / 2;
        
        m_objBigFont    = Font.getFont(Font.FACE_MONOSPACE, Font.STYLE_BOLD, Font.SIZE_LARGE);
        m_objNormalFont = Font.getFont(Font.FACE_PROPORTIONAL, Font.STYLE_PLAIN, Font.SIZE_MEDIUM);
        m_objSmallFont  = Font.getFont(Font.FACE_PROPORTIONAL, Font.STYLE_PLAIN, Font.SIZE_SMALL);
        m_nBigFontLineHeight    = m_objBigFont.getHeight();
        m_nNormalFontLineHeight = m_objNormalFont.getHeight();
        m_nSmallFontLineHeight  = m_objSmallFont.getHeight();
        
        m_nPositionRadius = m_objNormalFont.charsWidth("30".toCharArray(), 0, 2) * 13 / 10 / 2;
        m_nGaugeStartX = m_nPreviewStartX + 1;
        m_nGaugeWidth = m_nPreviewSize - 1;
        m_nGaugeHeight = 2;
        m_nPositionCenterX = m_nGaugeStartX + m_nGaugeWidth / 2;
        m_nPositionStartY =
            m_nAreaStartY +
            (m_nAreaHeight +
            m_nPreviewSize +
            m_nNormalFontLineHeight * 2 +
            m_nSmallFontLineHeight * 2) / 2 -
            m_nPositionRadius -
            m_nGaugeHeight;
        m_nNextPositionGaugeStartY = m_nPositionStartY + m_nPositionRadius * 2 + 1;
        m_nTotalGaugeStartY = m_nNextPositionGaugeStartY + m_nGaugeHeight + 1;
        
        m_bDrawPosition = (m_nTotalGaugeStartY + m_nGaugeHeight < m_nHeight &&
            m_nPositionStartY > m_nAreaStartY + m_nPreviewSize + m_nNormalFontLineHeight * 2 + m_nSmallFontLineHeight * 2);
        
        m_imgBrick00 = Image.createImage(m_nBrickSize + m_nBrickDepth, m_nBrickSize + m_nBrickDepth);
        m_imgBrick10 = Image.createImage(m_nBrickSize + m_nBrickDepth, m_nBrickSize + m_nBrickDepth);
        m_imgBrick01 = Image.createImage(m_nBrickSize + m_nBrickDepth, m_nBrickSize + m_nBrickDepth);
        m_imgBrick11 = Image.createImage(m_nBrickSize + m_nBrickDepth, m_nBrickSize + m_nBrickDepth);
        m_imgBrick02 = Image.createImage(m_nBrickSize + m_nBrickDepth, m_nBrickSize + m_nBrickDepth);
        m_imgBrick12 = Image.createImage(m_nBrickSize + m_nBrickDepth, m_nBrickSize + m_nBrickDepth);
        
        setCommandListener(this);
        localize();
    }
    
    public synchronized void clearSideMessage()
    {
        m_strSideMessage = "";
        repaint();
    }
    
    public void commandAction(Command       cmd,
                              Displayable   d)
    {
        if (m_objGame.getState() == TetrisGame.STATE_GAMEOVER)
            m_objApplication.confirmGameOver();
        else
            m_objApplication.enterMenu();
    }
    
    private void drawArea(Graphics  g)
    {
        int     x, y;
        int     n;
        
        for (y = TetrisGame.HEIGHT; --y >= 0;)
        {
            for (x = 0; x < TetrisGame.WIDTH; x++)
            {
                if (m_objGame.getBrick(x, y) != TetrisGame.NOBRICK)
                {
                    n = (m_objGame.getBrick(x - 1, y) == TetrisGame.NOBRICK) ? 0 : 1;
                    n += (m_objGame.getBrick(x, y + 1) == TetrisGame.NOBRICK) ? 0 : 2;
                    n += (m_objGame.getBrick(x + 1, y + 1) == TetrisGame.NOBRICK) ? 0 : 4;
                    switch (n)
                    {
                    case 0:
                        g.drawImage(
                            m_imgBrick00,
                            m_nAreaStartX + x * m_nBrickSize,
                            m_nAreaStartY + y * m_nBrickSize,
                            Graphics.TOP | Graphics.LEFT);
                        break;
                    case 1:
                        g.drawImage(
                            m_imgBrick10,
                            m_nAreaStartX + x * m_nBrickSize,
                            m_nAreaStartY + y * m_nBrickSize,
                            Graphics.TOP | Graphics.LEFT);
                        break;
                    case 2:
                        g.drawImage(
                            m_imgBrick01,
                            m_nAreaStartX + x * m_nBrickSize,
                            m_nAreaStartY + y * m_nBrickSize,
                            Graphics.TOP | Graphics.LEFT);
                        break;
                    case 3:
                        g.drawImage(
                            m_imgBrick11,
                            m_nAreaStartX + x * m_nBrickSize,
                            m_nAreaStartY + y * m_nBrickSize,
                            Graphics.TOP | Graphics.LEFT);
                        break;
                    case 4:
                    case 6:
                        g.drawImage(
                            m_imgBrick02,
                            m_nAreaStartX + x * m_nBrickSize,
                            m_nAreaStartY + y * m_nBrickSize,
                            Graphics.TOP | Graphics.LEFT);
                        break;
                    case 5:
                    case 7:
                        g.drawImage(
                            m_imgBrick12,
                            m_nAreaStartX + x * m_nBrickSize,
                            m_nAreaStartY + y * m_nBrickSize,
                            Graphics.TOP | Graphics.LEFT);
                        break;
                    }
                }
                else if (m_objGame.getBrickPreview(x, y) != 0)
                {
                    g.setColor(m_nBrickTopColor);
                    g.drawLine(
                        m_nAreaStartX + x * m_nBrickSize,
                        m_nAreaStartY + y * m_nBrickSize + m_nBrickDepth,
                        m_nAreaStartX + x * m_nBrickSize + m_nBrickDepth,
                        m_nAreaStartY + y * m_nBrickSize);
                    g.drawLine(
                        m_nAreaStartX + x * m_nBrickSize + m_nBrickDepth,
                        m_nAreaStartY + y * m_nBrickSize,
                        m_nAreaStartX + (x + 1) * m_nBrickSize + m_nBrickDepth,
                        m_nAreaStartY + y * m_nBrickSize);
                    g.drawLine(
                        m_nAreaStartX + x * m_nBrickSize,
                        m_nAreaStartY + y * m_nBrickSize + m_nBrickDepth,
                        m_nAreaStartX + (x + 1) * m_nBrickSize,
                        m_nAreaStartY + y * m_nBrickSize + m_nBrickDepth);
                    g.drawLine(
                        m_nAreaStartX + (x + 1) * m_nBrickSize,
                        m_nAreaStartY + y * m_nBrickSize + m_nBrickDepth,
                        m_nAreaStartX + (x + 1) * m_nBrickSize + m_nBrickDepth,
                        m_nAreaStartY + y * m_nBrickSize);

                    g.drawLine(
                        m_nAreaStartX + x * m_nBrickSize,
                        m_nAreaStartY + (y + 1) * m_nBrickSize + m_nBrickDepth,
                        m_nAreaStartX + x * m_nBrickSize + m_nBrickDepth,
                        m_nAreaStartY + (y + 1) * m_nBrickSize);
                    g.drawLine(
                        m_nAreaStartX + x * m_nBrickSize + m_nBrickDepth,
                        m_nAreaStartY + (y + 1) * m_nBrickSize,
                        m_nAreaStartX + (x + 1) * m_nBrickSize + m_nBrickDepth,
                        m_nAreaStartY + (y + 1) * m_nBrickSize);
                    g.drawLine(
                        m_nAreaStartX + x * m_nBrickSize,
                        m_nAreaStartY + (y + 1) * m_nBrickSize + m_nBrickDepth,
                        m_nAreaStartX + (x + 1) * m_nBrickSize,
                        m_nAreaStartY + (y + 1) * m_nBrickSize + m_nBrickDepth);
                    g.drawLine(
                        m_nAreaStartX + (x + 1) * m_nBrickSize,
                        m_nAreaStartY + (y + 1) * m_nBrickSize + m_nBrickDepth,
                        m_nAreaStartX + (x + 1) * m_nBrickSize + m_nBrickDepth,
                        m_nAreaStartY + (y + 1) * m_nBrickSize);
                    
                    g.drawLine(
                        m_nAreaStartX + x * m_nBrickSize,
                        m_nAreaStartY + y * m_nBrickSize + m_nBrickDepth,
                        m_nAreaStartX + x * m_nBrickSize,
                        m_nAreaStartY + (y + 1) * m_nBrickSize + m_nBrickDepth);
                    g.drawLine(
                        m_nAreaStartX + (x + 1) * m_nBrickSize,
                        m_nAreaStartY + y * m_nBrickSize + m_nBrickDepth,
                        m_nAreaStartX + (x + 1) * m_nBrickSize,
                        m_nAreaStartY + (y + 1) * m_nBrickSize + m_nBrickDepth);
                    g.drawLine(
                        m_nAreaStartX + x * m_nBrickSize + m_nBrickDepth,
                        m_nAreaStartY + y * m_nBrickSize,
                        m_nAreaStartX + x * m_nBrickSize + m_nBrickDepth,
                        m_nAreaStartY + (y + 1) * m_nBrickSize);
                    g.drawLine(
                        m_nAreaStartX + (x + 1) * m_nBrickSize + m_nBrickDepth,
                        m_nAreaStartY + y * m_nBrickSize,
                        m_nAreaStartX + (x + 1) * m_nBrickSize + m_nBrickDepth,
                        m_nAreaStartY + (y + 1) * m_nBrickSize);
                }
            }
        }
    }
    
    private void drawBrickToGraphics(Graphics   g,
                                     int        x,
                                     int        y)
    {
        int         i;
        
        g.setColor(m_nBrickFaceColor);
        g.fillRect(x, y + m_nBrickDepth + 1, x + m_nBrickSize - 1, y + m_nBrickSize - 1);
        
        g.setColor(m_nBrickTopColor);
        for (i = m_nBrickDepth + 1; --i >= 0;)
        {
            g.drawLine(
                x + m_nBrickDepth - i + 1,
                y + i,
                x + m_nBrickDepth - i + m_nBrickSize,
                y + i);
        }
        
        g.setColor(m_nBrickRightColor);
        for (i = m_nBrickDepth + 1; --i >= 0;)
        {
            g.drawLine(
                x + m_nBrickSize - 1 + i,
                y + m_nBrickSize + m_nBrickDepth - 1 - i,
                x + m_nBrickSize - 1 + i,
                y + m_nBrickDepth + 1 - i);
        }
    }
    
    private void drawPreview(Graphics   g)
    {
        int     x, y;
        int     n;
        
        for (y = 4; --y >= 0;)
        {
            for (x = 0; x < 4; x++)
            {
                if (m_objGame.getPreviewBrick(x, y) != TetrisGame.NOBRICK)
                {
                    n = (m_objGame.getPreviewBrick(x - 1, y) == TetrisGame.NOBRICK) ? 0 : 1;
                    n += (m_objGame.getPreviewBrick(x, y + 1) == TetrisGame.NOBRICK) ? 0 : 2;
                    n += (m_objGame.getPreviewBrick(x + 1, y + 1) == TetrisGame.NOBRICK) ? 0 : 4;
                    switch (n)
                    {
                    case 0:
                        g.drawImage(
                            m_imgBrick00,
                            m_nPreviewStartX + x * m_nBrickSize,
                            m_nAreaStartY + y * m_nBrickSize,
                            Graphics.TOP | Graphics.LEFT);
                        break;
                    case 1:
                        g.drawImage(
                            m_imgBrick10,
                            m_nPreviewStartX + x * m_nBrickSize,
                            m_nAreaStartY + y * m_nBrickSize,
                            Graphics.TOP | Graphics.LEFT);
                        break;
                    case 2:
                        g.drawImage(
                            m_imgBrick01,
                            m_nPreviewStartX + x * m_nBrickSize,
                            m_nAreaStartY + y * m_nBrickSize,
                            Graphics.TOP | Graphics.LEFT);
                        break;
                    case 3:
                        g.drawImage(
                            m_imgBrick11,
                            m_nPreviewStartX + x * m_nBrickSize,
                            m_nAreaStartY + y * m_nBrickSize,
                            Graphics.TOP | Graphics.LEFT);
                        break;
                    case 4:
                    case 6:
                        g.drawImage(
                            m_imgBrick02,
                            m_nPreviewStartX + x * m_nBrickSize,
                            m_nAreaStartY + y * m_nBrickSize,
                            Graphics.TOP | Graphics.LEFT);
                        break;
                    case 5:
                    case 7:
                        g.drawImage(
                            m_imgBrick12,
                            m_nPreviewStartX + x * m_nBrickSize,
                            m_nAreaStartY + y * m_nBrickSize,
                            Graphics.TOP | Graphics.LEFT);
                        break;
                    }
                }
            }
        }
    }
    
    private void drawStatus(Graphics    g,
                            String      strMessage,
                            int         nMessageColor)
    {
        int nMessageWidth;
        
        nMessageWidth = m_objBigFont.charsWidth(
            strMessage.toCharArray(),
            0,
            strMessage.length());
        g.setColor(m_nBackgroundColor);
        g.fillRect(
            3,
            m_nHeight - 4 - 3 - m_nBigFontLineHeight - 3,
            3 + nMessageWidth + 3,
            3 + m_nBigFontLineHeight + 3);
        g.setColor(m_nBorderColor);
        g.drawRect(
            3,
            m_nHeight - 4 - 3 - m_nBigFontLineHeight - 3,
            3 + nMessageWidth + 3,
            3 + m_nBigFontLineHeight + 3);
        g.setColor(nMessageColor);
        g.setFont(m_objBigFont);
        g.drawString(strMessage, 3 + 3, m_nHeight - 4 - 3 - m_nBigFontLineHeight, Graphics.LEFT | Graphics.TOP);
    }
    
    private void initializeBackground()
    {
        int         i, j;
        int         nBaseLineColor;
        int         nRandom;
        Random      objRandom       = new Random();
        int         nColorComponent;
        Graphics    g;
        
        m_imgBackground = Image.createImage(m_nWidth, m_nHeight);
        g = m_imgBackground.getGraphics();
        g.drawImage(
            m_objColorThemeKeeper.getBackgroundImage(),
            0, 0,
            Graphics.LEFT | Graphics.TOP);

        g.setColor(m_nBackgroundColor);
        g.fillRect(
            m_nPreviewStartX - 2,
            m_nAreaStartY - 2,
            m_nPreviewSize + 4,
            m_nPreviewSize + 4);
        g.fillRect(
            m_nAreaStartX - 2,
            m_nAreaStartY - 2,
            m_nAreaWidth + 4,
            m_nAreaHeight + 4);
        g.setColor(m_nBorderColor);
        g.drawRect(
            m_nPreviewStartX - 2,
            m_nAreaStartY - 2,
            m_nPreviewSize + 4,
            m_nPreviewSize + 4);
        g.drawRect(
            m_nAreaStartX - 2,
            m_nAreaStartY - 2,
            m_nAreaWidth + 4,
            m_nAreaHeight + 4);
        if (m_bDrawPosition)
        {
            g.drawRoundRect(
                m_nGaugeStartX - 1,
                m_nNextPositionGaugeStartY - 1,
                m_nGaugeWidth + 1,
                m_nGaugeHeight + 1,
                2,
                2);
            g.drawRoundRect(
                m_nGaugeStartX - 1,
                m_nTotalGaugeStartY - 1,
                m_nGaugeWidth + 1,
                m_nGaugeHeight + 1,
                2,
                2);
        }

        g.setColor(m_nStatisticsColor);
        g.setFont(m_objSmallFont);
    }
    
    private void initializeBrickImages()
    {
        int         i;
        Graphics    g;

        g = m_imgBrick00.getGraphics();
        g.setColor(m_nBackgroundColor);
        g.fillRect(0, 0, m_nBrickSize + m_nBrickDepth, m_nBrickSize + m_nBrickDepth);
        drawBrickToGraphics(g, 0, 0);
        
        g = m_imgBrick10.getGraphics();
        g.setColor(m_nBackgroundColor);
        g.fillRect(0, 0, m_nBrickSize + m_nBrickDepth, m_nBrickSize + m_nBrickDepth);
        drawBrickToGraphics(g, -m_nBrickSize, 0);
        drawBrickToGraphics(g, 0, 0);
        
        g = m_imgBrick01.getGraphics();
        g.setColor(m_nBackgroundColor);
        g.fillRect(0, 0, m_nBrickSize + m_nBrickDepth, m_nBrickSize + m_nBrickDepth);
        drawBrickToGraphics(g, 0, m_nBrickSize);
        drawBrickToGraphics(g, 0, 0);

        g = m_imgBrick11.getGraphics();
        g.setColor(m_nBackgroundColor);
        g.fillRect(0, 0, m_nBrickSize + m_nBrickDepth, m_nBrickSize + m_nBrickDepth);
        drawBrickToGraphics(g, -m_nBrickSize, 0);
        drawBrickToGraphics(g, 0, m_nBrickSize);
        drawBrickToGraphics(g, 0, 0);
        
        g = m_imgBrick02.getGraphics();
        g.setColor(m_nBackgroundColor);
        g.fillRect(0, 0, m_nBrickSize + m_nBrickDepth, m_nBrickSize + m_nBrickDepth);
        drawBrickToGraphics(g, m_nBrickSize - 1, m_nBrickSize);
        drawBrickToGraphics(g, m_nBrickSize, m_nBrickSize);
        drawBrickToGraphics(g, 0, 0);
        
        g = m_imgBrick12.getGraphics();
        g.setColor(m_nBackgroundColor);
        g.fillRect(0, 0, m_nBrickSize + m_nBrickDepth, m_nBrickSize + m_nBrickDepth);
        drawBrickToGraphics(g, -m_nBrickSize, 0);
        drawBrickToGraphics(g, m_nBrickSize - 1, m_nBrickSize);
        drawBrickToGraphics(g, m_nBrickSize, m_nBrickSize);
        drawBrickToGraphics(g, 0, 0);
    }
    
    private void initializeColors()
    {
        m_nBackgroundColor      = m_objColorThemeKeeper.getBackgroundColor();
        m_nStatisticsColor      = m_objColorThemeKeeper.getTextColor();
        m_nBorderColor          = m_objColorThemeKeeper.getBorderColor();
        m_nGameOverMessageColor = m_objColorThemeKeeper.getBadMessageColor();
        m_nPauseMessageColor    = m_objColorThemeKeeper.getGoodMessageColor();
        m_nGaugeColor           = m_objColorThemeKeeper.getGaugeColor();
        m_nSideMessageColor     = m_objColorThemeKeeper.getSideMessageColor();
    }
    
    private void keyEvent(int nKeyCode, boolean bRepeated)
    {
        boolean bRepaint = false;
        
        if (m_objGame.getState() == TetrisGame.STATE_GAMEOVER)
            m_objApplication.confirmGameOver();

        switch (nKeyCode)
        {
            case KEY_NUM4:
            case LEFT:
            case -3:
                bRepaint = m_objGame.moveFigure(true);
                break;
                
            case KEY_NUM6:
            case RIGHT:
            case -4:
                bRepaint = m_objGame.moveFigure(false);
                break;
                
            case KEY_NUM5:
            case UP:
            case -1:
                if (!bRepeated)
                    bRepaint = m_objGame.rotateFigure();
                break;
                
            case KEY_NUM8:
            case DOWN:
            case -2:
                if (!bRepeated)
                {
                    m_objGame.dropFigure();
                    bRepaint = true;
                }
                break;
                
            case KEY_NUM1:
            case KEY_NUM3:
            case KEY_NUM7:
            case KEY_NUM9:
                if (!bRepeated)
                {
                    m_objGame.pauseOrResume();
                    bRepaint = true;
                }
                break;
                
            case KEY_STAR:  //  Quieter
                m_objSideMessageTicker.startTimer();
                m_strSideMessage =
                    m_objLocalizer.get(Localizer.S_VOLUME_IS) + " " +
                    m_objApplication.hotKeyQuieter() + "%";
                bRepaint = true;
                break;

            case KEY_POUND:  //  Louder
                m_objSideMessageTicker.startTimer();
                m_strSideMessage =
                    m_objLocalizer.get(Localizer.S_VOLUME_IS) + " " +
                    m_objApplication.hotKeyLouder() + "%";
                bRepaint = true;
                break;
                
            case KEY_NUM0:  //  Mute
                m_objSideMessageTicker.startTimer();
                m_strSideMessage =
                    m_objLocalizer.get(
                        m_objApplication.hotKeyMute() ?
                        Localizer.S_MUTE_ON : Localizer.S_MUTE_OFF);
                bRepaint = true;
                break;
        }
        if (bRepaint)
            repaint();
    }
    
    protected void keyPressed(int nKeyCode)
    {
        keyEvent(nKeyCode, false);
    }
    
    protected void keyRepeated(int nKeyCode)
    {
        keyEvent(nKeyCode, true);
    }
    
    public void localize()
    {
        if (m_cmdMenu != null)
            removeCommand(m_cmdMenu);
        m_cmdMenu = new Command(m_objLocalizer.get(Localizer.S_MENU), Command.SCREEN, 0);
        addCommand(m_cmdMenu);
    }
    
    public void onNewGame()
    {
        if (m_imgLehasoftBanner != null)
        {
            if (m_objRandom.nextInt() % 10 == 0)
                m_imgCurrentBanner = m_imgGetSoftBanner;
            else
                m_imgCurrentBanner = m_imgLehasoftBanner;
        }
    }
    
    protected synchronized void paint(Graphics   g)
    {
        int     nScores = m_objGame.getScores();
        int     nPosition;
        int     nLeaderScores;
        int     nNextPositionScores;
        int     nPreviousPositionScores;
        int     nNextPositionGaugeWidth;
        int     nTotalGaugeWidth;
        
        g.drawImage(m_imgBackground, 0, 0, Graphics.TOP | Graphics.LEFT);
        if (m_imgCurrentBanner != null)
        {
            g.drawImage(
                m_imgCurrentBanner,
                m_nBannerStartX,
                m_nBannerStartY,
                Graphics.TOP | Graphics.LEFT);
        }

        drawArea(g);
        drawPreview(g);
        if (m_bDrawPosition)
        {
            g.setColor(m_nBackgroundColor);
            g.fillArc(
                m_nPositionCenterX - m_nPositionRadius,
                m_nPositionStartY,
                m_nPositionRadius * 2,
                m_nPositionRadius * 2,
                0,
                360);
            g.fillRect(
                m_nGaugeStartX,
                m_nNextPositionGaugeStartY,
                m_nGaugeWidth,
                m_nGaugeHeight);
            g.fillRect(
                m_nGaugeStartX,
                m_nTotalGaugeStartY,
                m_nGaugeWidth,
                m_nGaugeHeight);
            g.setColor(m_nBorderColor);
            g.drawArc(
                m_nPositionCenterX - m_nPositionRadius,
                m_nPositionStartY,
                m_nPositionRadius * 2,
                m_nPositionRadius * 2,
                0,
                360);
        }
        g.setColor(m_nStatisticsColor);
        g.setFont(m_objSmallFont);
        g.drawString(
            m_objLocalizer.get(Localizer.S_SCORES),
            m_nPreviewStartX,
            m_nAreaStartY + m_nPreviewSize + m_nBrickDepth + 5,
            Graphics.LEFT | Graphics.TOP);
        g.drawString(
            m_objLocalizer.get(Localizer.S_LINES),
            m_nPreviewStartX,
            m_nAreaStartY + m_nPreviewSize + m_nBrickDepth + 5 + m_nSmallFontLineHeight + m_nNormalFontLineHeight,
            Graphics.LEFT | Graphics.TOP);

        g.setFont(m_objNormalFont);
        g.drawString(
            String.valueOf(nScores),
            m_nPreviewStartX,
            m_nAreaStartY + m_nPreviewSize + 5 + m_nSmallFontLineHeight,
            Graphics.LEFT | Graphics.TOP);
        g.drawString(
            String.valueOf(m_objGame.getLinesCount()),
            m_nPreviewStartX,
            m_nAreaStartY + m_nPreviewSize + 5 + m_nSmallFontLineHeight * 2 + m_nNormalFontLineHeight,
            Graphics.LEFT | Graphics.TOP);
        
        if (m_bDrawPosition)
        {
            nPosition = m_objGame.getScoresPosition();
            if (nPosition != 0)
            {
                g.drawString(
                    String.valueOf(nPosition),
                    m_nPositionCenterX,
                    m_nPositionStartY + m_nPositionRadius - m_nNormalFontLineHeight / 2,
                    Graphics.HCENTER | Graphics.TOP);
            }
            if (nPosition == 1)
            {
                nNextPositionGaugeWidth = m_nGaugeWidth;
                nTotalGaugeWidth = m_nGaugeWidth;
            }
            else
            {
                nLeaderScores           = m_objGame.getLeaderScores();
                nNextPositionScores     = m_objGame.getNextPositionScores();
                nPreviousPositionScores = m_objGame.getPreviousPositionScores();
                if (nNextPositionScores == nPreviousPositionScores)
                    nNextPositionGaugeWidth = 0;
                else
                {
                    nNextPositionGaugeWidth =
                        (nScores - nPreviousPositionScores) * m_nGaugeWidth /
                        (nNextPositionScores - nPreviousPositionScores);
                }
                if (nLeaderScores == 0)
                    nTotalGaugeWidth = 0;
                else
                    nTotalGaugeWidth = nScores * m_nGaugeWidth / nLeaderScores;
            }
            g.setColor(m_nGaugeColor);
            g.fillRect(
                m_nGaugeStartX,
                m_nNextPositionGaugeStartY,
                nNextPositionGaugeWidth,
                m_nGaugeHeight);
            g.fillRect(
                m_nGaugeStartX,
                m_nTotalGaugeStartY,
                nTotalGaugeWidth,
                m_nGaugeHeight);
        }
        
        if (!m_strSideMessage.equals(""))
            drawStatus(g, m_strSideMessage, m_nSideMessageColor);
        else
        {
            switch (m_objGame.getState())
            {
                case TetrisGame.STATE_GAMEOVER:
                    drawStatus(g, m_objLocalizer.get(Localizer.S_GAME_OVER), m_nGameOverMessageColor);
                    break;
                case TetrisGame.STATE_PAUSED:
                    drawStatus(g, m_objLocalizer.get(Localizer.S_PAUSE), m_nPauseMessageColor);
            }
        }
    }
    
    public void setBricksColor(int  nColor)
    {
        int nRed;
        int nGreen;
        int nBlue;
        
        m_nBrickFaceColor = nColor;
        nRed    = Color.red(nColor);
        nGreen  = Color.green(nColor);
        nBlue   = Color.blue(nColor);
        m_nBrickTopColor = Color.fromRGB(
            nRed / 2,
            nGreen / 2,
            nBlue / 2);
        m_nBrickRightColor = Color.fromRGB(
            127 + nRed / 2,
            127 + nGreen / 2,
            127 + nBlue / 2);
        
        initializeBrickImages();
    }
    
    public void setColorThemeKeeperObject(ColorThemeKeeper  objColorThemeKeeper)
    {
        m_objColorThemeKeeper = objColorThemeKeeper;
        initializeColors();
        initializeBackground();
    }
    
    public void setGameObject(TetrisGame    objGame)
    {
        m_objGame = objGame;
    }
}