/* Lehasoft Tetris Mobile 2006
   Copyright (c) Alexey Inkin, Lehasoft (1996-2006), GetSoft.ru, leha@inkin.ru

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  */

package Main;

import java.util.*;

public class Localizer
{
    private int                 m_nLocale;
    private SettingsKeeper      m_objSettingsKeeper;
    private LanguageInformation m_arrLanguageInformation[];
    private Vector              m_arrLocalizable            = new Vector();
    private Hashtable           m_english                   = new Hashtable();
    private Hashtable           m_russian                   = new Hashtable();
    private Hashtable           m_currentHash;
    
    public final static String S_ABOUT          = "ABOUT";
    public final static String S_BACK           = "BACK";
    public final static String S_BRICK_COLOR    = "BRICK_COLOR";
    public final static String S_CANCEL         = "CANCEL";
    public final static String S_CLEAR          = "CLEAR";
    public final static String S_CONTROL_KEYS   = "CONTROL_KEYS";
    public final static String S_CONTROL_KEYS_CONTENT   = "CONTROL_KEYS_CONTENT";
    public final static String S_ENTER_NAME     = "ENTER_NAME";
    public final static String S_EXIT           = "EXIT";
    public final static String S_GAME_OVER      = "GAME_OVER";
    public final static String S_HIGH_SCORES    = "HIGH_SCORES";
    public final static String S_LINES          = "LINES";
    public final static String S_LANGUAGE       = "LANGUAGE";
    public final static String S_MENU           = "MENU";
    public final static String S_MUTE           = "MUTE";
    public final static String S_MUTE_OFF       = "MUTE_OFF";
    public final static String S_MUTE_ON        = "MUTE_ON";
    public final static String S_NO_RECORDS     = "NO_RECORDS";
    public final static String S_PAUSE          = "PAUSE";
    public final static String S_PREFERENCES    = "PREFERENCES";
    public final static String S_RESET          = "RESET";
    public final static String S_RESTART        = "RESTART";
    public final static String S_RESUME         = "RESUME";
    public final static String S_SCORES         = "SCORES";
    public final static String S_SOUND          = "SOUND";
    public final static String S_START          = "START";
    public final static String S_VOLUME         = "VOLUME";
    public final static String S_VOLUME_IS      = "VOLUME_IS";

    public Localizer(SettingsKeeper sk)
    {
        m_objSettingsKeeper = sk;
        m_arrLanguageInformation = new LanguageInformation[] {
            new LanguageInformation("English", "en-GB", LanguageInformation.ENGLISH),
            new LanguageInformation("Russian", "ru-RU", LanguageInformation.RUSSIAN)};
            
        loadLocale();
        
        m_english.put(S_ABOUT, "About");
        m_russian.put(S_ABOUT, "Об Игре");
        
        m_english.put(S_BACK, "Back");
        m_russian.put(S_BACK, "Назад");
        
        m_english.put(S_BRICK_COLOR, "Brick Color");
        m_russian.put(S_BRICK_COLOR, "Цвет Кирпичей");

        m_english.put(S_CANCEL, "Cancel");
        m_russian.put(S_CANCEL, "Отмена");
        
        m_english.put(S_CLEAR, "Clear");
        m_russian.put(S_CLEAR, "Очистить");
        
        m_english.put(S_CONTROL_KEYS, "Control Keys");
        m_russian.put(S_CONTROL_KEYS, "Управление");
        
        m_english.put(S_CONTROL_KEYS_CONTENT,
            "Left, 4 - Move Left\n" +
            "Right, 6 - Move Right\n" +
            "Up, 5 - Rotate\n" +
            "Down, 8 - Drop\n" +
            "1, 3, 7, 9 - Pause / Resume\n" +
            "# - Louder\n" +
            "* - Quieter\n" +
            "0 - Mute / Unmute");
        m_russian.put(S_CONTROL_KEYS_CONTENT,
            "Влево, 4 - Сдвинуть Влево\n" +
            "Вправо, 6 - Сдвинуть Вправо\n" +
            "Вверх, 5 - Повернуть\n" +
            "Вниз, 8 - Сбросить\n" +
            "1, 3, 7, 9 - Пауза / Продолжить\n" +
            "# - Громче\n" +
            "* - Тише\n" +
            "0 - Вкл/Выкл Звук");
        
        m_english.put(S_ENTER_NAME, "Enter Your Name");
        m_russian.put(S_ENTER_NAME, "Введите Ваше Имя");
        
        m_english.put(S_EXIT, "Exit");
        m_russian.put(S_EXIT, "Выйти");
        
        m_english.put(S_GAME_OVER, "GAME OVER");
        m_russian.put(S_GAME_OVER, "КОНЕЦ");
        
        m_english.put(S_HIGH_SCORES, "High Scores");
        m_russian.put(S_HIGH_SCORES, "Лидеры");
        
        m_english.put(S_LINES, "lines:");
        m_russian.put(S_LINES, "ряды:");
        
        m_english.put(S_LANGUAGE, "Language");
        m_russian.put(S_LANGUAGE, "Язык");
        
        m_english.put(S_MENU, "Menu");
        m_russian.put(S_MENU, "Меню");
        
        m_english.put(S_MUTE, "Mute");
        m_russian.put(S_MUTE, "Без Звука");
        
        m_english.put(S_MUTE_OFF, "MUTE OFF");
        m_russian.put(S_MUTE_OFF, "ЗВУК ВКЛ");
        
        m_english.put(S_MUTE_ON, "MUTE ON");
        m_russian.put(S_MUTE_ON, "БЕЗ ЗВУКА");
        
        m_english.put(S_NO_RECORDS, "No Records");
        m_russian.put(S_NO_RECORDS, "Нет Записей");
        
        m_english.put(S_PAUSE, "PAUSE");
        m_russian.put(S_PAUSE, "ПАУЗА");
        
        m_english.put(S_PREFERENCES, "Preferences");
        m_russian.put(S_PREFERENCES, "Настройки");
        
        m_english.put(S_RESET, "Reset");
        m_russian.put(S_RESET, "Сбросить");
        
        m_english.put(S_RESTART, "Restart");
        m_russian.put(S_RESTART, "Заново");
        
        m_english.put(S_RESUME, "Resume");
        m_russian.put(S_RESUME, "Возобновить");
        
        m_english.put(S_SCORES, "scores:");
        m_russian.put(S_SCORES, "очки:");
        
        m_english.put(S_SOUND, "Sound");
        m_russian.put(S_SOUND, "Звуки");
        
        m_english.put(S_START, "Start");
        m_russian.put(S_START, "Начать");
        
        m_english.put(S_VOLUME, "Volume");
        m_russian.put(S_VOLUME, "Громкость");
        
        m_english.put(S_VOLUME_IS, "VOLUME:");
        m_russian.put(S_VOLUME_IS, "ГРОМКОСТЬ:");
    }
    
    public void addLocalizable(Localizable l)
    {
        m_arrLocalizable.addElement(l);
    }
    
    public String get(String strKey)
    {
        return (String)m_currentHash.get(strKey);
    }
    
    public LanguageInformation[] getLanguageList()
    {
        return m_arrLanguageInformation;
    }
    
    public int getLocale()
    {
        return m_nLocale;
    }
    
    public String getPositionText(int nPosition)
    {
        String strResult;
        
        switch (m_nLocale)
        {
            case LanguageInformation.RUSSIAN:
                strResult = "Вы заняли " + String.valueOf(nPosition) + "-е место. Введите ваше имя:";
                break;
                
            default:
                String strPosition;
                
                switch (nPosition)
                {
                    case 1:
                        strPosition = "1st";
                        break;
                    case 2:
                        strPosition = "2nd";
                        break;
                    case 3:
                        strPosition = "3rd";
                        break;
                    default:
                        strPosition = String.valueOf(nPosition) + "th";
                }
                strResult = "You took " + strPosition + " position. Enter your name:";
        }
        return strResult;
    }
    
    private void loadLocale()
    {
        String strLocale;
        
        strLocale = m_objSettingsKeeper.getLocale();
        if (strLocale == "")
            strLocale = java.lang.System.getProperty("microedition.locale");
        
        if (strLocale.equals("ru-RU"))
            setLocale(LanguageInformation.RUSSIAN);
        else
            setLocale(LanguageInformation.ENGLISH);
    }
    
    public void setLocale(int nLocale)
    {
        Enumeration e;
        String      strLocaleToSave = "";
        
        m_nLocale = nLocale;
        
        if (nLocale == LanguageInformation.RUSSIAN)
        {
            m_currentHash = m_russian;
            strLocaleToSave = "ru-RU";
        }
        else
        {
            m_currentHash = m_english;
            strLocaleToSave = "en-GB";  //  USA? No way! :)
        }
        
        e = m_arrLocalizable.elements();
        while (e.hasMoreElements())
            ((Localizable)e.nextElement()).localize();
    }
}