/* Lehasoft Tetris Mobile 2006
   Copyright (c) Alexey Inkin, Lehasoft (1996-2006), GetSoft.ru, leha@inkin.ru

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  */

package Main;

import Game.ScoresTable;
import Game.ScoresTableRecord;

public class SettingsKeeper
{
    private ParamsKeeper    m_objParamsKeeper;
    private int             m_nScoresTableRecordCount;
    
    private final static String PARAM_BRICKS_COLOR      = "BricksColor";
    private final static String PARAM_LOCALE            = "Locale";
    private final static String PARAM_MUTE              = "Mute";
    private final static String PARAM_SCORES_TABLE      = "ScoresTable";
    private final static String PARAM_VOLUME            = "Volume";
    
    private final static String DEFAULT_BRICKS_COLOR    = "16744544";
    private final static String DEFAULT_VOLUME          = "50";

    public SettingsKeeper()
    {
        m_objParamsKeeper = new ParamsKeeper();
    }
    
    public void clearScoresTable()
    {
        m_objParamsKeeper.deleteParam(PARAM_SCORES_TABLE);
    }
    
    public int getBricksColor()
    {
        return Integer.parseInt(m_objParamsKeeper.getParam(
            PARAM_BRICKS_COLOR,
            DEFAULT_BRICKS_COLOR));
    }
    
    public String getLocale()
    {
        return m_objParamsKeeper.getParam(PARAM_LOCALE, "");
    }
    
    public boolean getMute()
    {
        return Integer.parseInt(m_objParamsKeeper.getParam(PARAM_MUTE, "0")) == 1;
    }
    
    public int getScoresTableRecordCount()
    {
        return m_nScoresTableRecordCount;
    }
    
    public int getVolume()
    {
        return Integer.parseInt(m_objParamsKeeper.getParam(
            PARAM_VOLUME,
            DEFAULT_VOLUME));
    }
    
    public ScoresTableRecord[] loadScoresTable()
    {
        int     i;
        String  strScoresList   = m_objParamsKeeper.getParam(PARAM_SCORES_TABLE, "");
        String  arrScores[]     = splitString(strScoresList, '\0');
        int     nScores;
        ScoresTableRecord   arrResult[] = new ScoresTableRecord[ScoresTable.RECORDS_COUNT];
        
        m_nScoresTableRecordCount = arrScores.length / 2;
        for (i = m_nScoresTableRecordCount; --i >= 0;)
        {
            nScores = Integer.parseInt(arrScores[i * 2 + 1]);
            arrResult[i] = new ScoresTableRecord(arrScores[i * 2], nScores);
        }
        
        return arrResult;
    }
    
    public void saveScoresTable(ScoresTableRecord arrScoresTable[])
    {
        int     i;
        String  strScoresList = "";
        
        for (i = 0; i < arrScoresTable.length && arrScoresTable[i] != null; i++)
        {
            strScoresList +=
                "\0" + arrScoresTable[i].getName() +
                "\0" + arrScoresTable[i].getScores();
        }

        strScoresList = strScoresList.substring(1);
        m_objParamsKeeper.setParam(PARAM_SCORES_TABLE, strScoresList);
    }
    
    public void setBricksColor(int nColor)
    {
        m_objParamsKeeper.setParam(PARAM_BRICKS_COLOR, String.valueOf(nColor));
    }
    
    public void setLocale(String strLocale)
    {
        m_objParamsKeeper.setParam(PARAM_LOCALE, strLocale);
    }
    
    public void setMute(boolean b)
    {
        m_objParamsKeeper.setParam(
            PARAM_MUTE,
            String.valueOf(b ? 1 : 0));
    }
    
    public void setVolume(int n)
    {
        m_objParamsKeeper.setParam(PARAM_VOLUME, String.valueOf(n));
    }
    
    public static String[] splitString(String str, char cDelimiter)
    {
        if (str.length() == 0)
            return new String[0];
        
        int     i;
        int     nStart = 0;
        int     nCurrentChunk = 0;
        int     nChunkCount = 1;
        char    arrChars[] = str.toCharArray();
        String  arrResult[];

        for (i = arrChars.length; --i >= 0;)
        {
            if (arrChars[i] == cDelimiter)
                nChunkCount++;
        }
        
        arrResult = new String[nChunkCount];
        
        for (i = 0; nCurrentChunk < nChunkCount - 1; i++)
        {
            if (arrChars[i] == cDelimiter)
            {
                arrResult[nCurrentChunk++] = str.substring(nStart, i);
                nStart = i + 1;
            }
        }
        arrResult[nChunkCount - 1] = str.substring(i);
        
        return arrResult;
    }
}