/* Lehasoft Tetris Mobile 2006
   Copyright (c) Alexey Inkin, Lehasoft (1996-2006), GetSoft.ru, leha@inkin.ru

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  */

package Main;

import Game.*;
import Interface.*;
import java.util.*;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;

public class Tetris extends MIDlet
{
    private Display             m_objDisplay;
    private TetrisGame          m_objGame;
    private Localizer           m_objLocalizer;
    private SettingsKeeper      m_objSettingsKeeper     = new SettingsKeeper();
    private ColorThemeKeeper    m_objColorThemeKeeper;
    private TetrisCanvas        m_objCanvas;
    private AboutCanvas         m_objAboutCanvas;
    private MainMenu            m_lstMenu;
    private LanguageMenu        m_lstLanguages;
    private PreferencesMenu     m_lstPreferences;
    private ColorDialogue       m_objColorDialogue;
    private ControlDialogue     m_objControlDialogue;
    private SoundDialogue       m_objSoundDialogue;
    private ScoresTable         m_objScoresTable;
    private UserNameInput       m_objUserNameInput;
    private ScoresTableCanvas   m_objScoresTableCanvas;
    private Stack               m_objDisplayableStack   = new Stack();

    private int                 m_nScores;
    
    private final static int BRICK_COLOR    = 0;
    
    public Tetris()
    {
        int nBricksColor;

        m_objDisplay = Display.getDisplay(this);
        Alerter.setDisplay(m_objDisplay);
        
        m_objLocalizer          = new Localizer(m_objSettingsKeeper);
        m_objGame               = new TetrisGame(this, m_objSettingsKeeper);
        m_objCanvas             = new TetrisCanvas(this, m_objLocalizer);
        m_objAboutCanvas        = new AboutCanvas(this);
        m_objColorDialogue      = new ColorDialogue(this, m_objLocalizer);
        m_objControlDialogue    = new ControlDialogue(this, m_objLocalizer);
        m_objSoundDialogue      = new SoundDialogue(this, m_objLocalizer, m_objSettingsKeeper);
        m_objScoresTable        = new ScoresTable(m_objSettingsKeeper);
        m_objUserNameInput      = new UserNameInput(this, m_objLocalizer);
        m_objScoresTableCanvas  = new ScoresTableCanvas(this, m_objLocalizer);
        
        m_objDisplay.setCurrent(m_objCanvas);
        m_objColorThemeKeeper = new ColorThemeKeeper();
        m_objColorThemeKeeper.setDisplayObject(m_objDisplay);
        m_objColorThemeKeeper.setSize(m_objCanvas.getWidth(), m_objCanvas.getHeight());

        m_objGame.setScoresTableObject(m_objScoresTable);
        
        m_objCanvas.setGameObject(m_objGame);
        m_objCanvas.setColorThemeKeeperObject(m_objColorThemeKeeper);

        m_objAboutCanvas.setColorThemeKeeperObject(m_objColorThemeKeeper);
        
        m_objScoresTableCanvas.setColorThemeKeeperObject(m_objColorThemeKeeper);
        m_objScoresTableCanvas.setScoresTableObject(m_objScoresTable);
        
        nBricksColor = m_objSettingsKeeper.getBricksColor();
        m_objColorDialogue.setColorThemeKeeperObject(m_objColorThemeKeeper);
        m_objColorDialogue.setColor(nBricksColor);
        m_objCanvas.setBricksColor(nBricksColor);
        
        m_lstMenu           = new MainMenu(this, m_objLocalizer);
        m_lstLanguages      = new LanguageMenu(this, m_objLocalizer);
        m_lstPreferences    = new PreferencesMenu(this, m_objLocalizer);

        m_objLocalizer.addLocalizable(m_objColorDialogue);
        m_objLocalizer.addLocalizable(m_objControlDialogue);
        m_objLocalizer.addLocalizable(m_lstLanguages);
        m_objLocalizer.addLocalizable(m_lstMenu);
        m_objLocalizer.addLocalizable(m_lstPreferences);
        m_objLocalizer.addLocalizable(m_objScoresTableCanvas);
        m_objLocalizer.addLocalizable(m_objSoundDialogue);
        m_objLocalizer.addLocalizable(m_objCanvas);
        m_objLocalizer.addLocalizable(m_objUserNameInput);
        
        m_objCanvas.onNewGame();
        m_objGame.start();
    }
    
    public void addUserNameToScoresTable(String strName)
    {
        m_objScoresTable.addTableRecord(strName, m_nScores);
        enterMenu();
    }
    
    public void confirmGameOver()
    {
        int nPosition;
        
        m_nScores = m_objGame.getScores();
        nPosition = m_objScoresTable.getScoresPosition(m_nScores);
        if (nPosition == 0)
            enterMenu();
        else
        {
            m_objUserNameInput.setScoresPosition(nPosition);
            m_objDisplay.setCurrent(m_objUserNameInput);
        }
        m_objGame.reset();
    }
    
    public void destroyApp(boolean  bUnconditional)
    {
    }
    
    public void enterMenu()
    {
        switch (m_objGame.getState())
        {
        case TetrisGame.STATE_ACTIVE:
            m_objGame.pause();
            m_lstMenu.showGamePaused();
            break;
        case TetrisGame.STATE_PAUSED:
            m_lstMenu.showGamePaused();
            break;
        default:
            m_lstMenu.showGameNotStarted();
        }
        m_objDisplay.setCurrent(m_lstMenu);
    }
    
    public void exit()
    {
        destroyApp(true);
        notifyDestroyed();
    }
    
    public String getColorDialogueTitle(int nParam)
    {
        switch (nParam)
        {
            case BRICK_COLOR:
                return m_objLocalizer.get(Localizer.S_BRICK_COLOR);
        }
        return "";
    }
    
    public int hotKeyLouder()
    {
        return m_objSoundDialogue.hotKeyLouder();
    }
    
    public boolean hotKeyMute()
    {
        return m_objSoundDialogue.hotKeyMute();
    }
    
    public int hotKeyQuieter()
    {
        return m_objSoundDialogue.hotKeyQuieter();
    }

    public void onAbout(Displayable d)
    {
        m_objDisplayableStack.push(d);
        m_objAboutCanvas.initialize();
        m_objDisplay.setCurrent(m_objAboutCanvas);
    }
    
    public void onBack()
    {
        m_objDisplay.setCurrent((Displayable)m_objDisplayableStack.pop());
    }
    
    public void onBricksColor(Displayable d)
    {
        m_objDisplayableStack.push(d);
        m_objDisplay.setCurrent(m_objColorDialogue);
    }
    
    public void onControlKeys(Displayable d)
    {
        m_objDisplayableStack.push(d);
        m_objDisplay.setCurrent(m_objControlDialogue);
    }
    
    public void onLanguage(Displayable d)
    {
        m_objDisplayableStack.push(d);
        m_lstLanguages.preShow();
        m_objDisplay.setCurrent(m_lstLanguages);
    }
    
    public void onPreferences(Displayable d)
    {
        m_objDisplayableStack.push(d);
        m_objDisplay.setCurrent(m_lstPreferences);
    }
    
    public void onScoresTable(Displayable d)
    {
        m_objDisplayableStack.push(d);
        m_objDisplay.setCurrent(m_objScoresTableCanvas);
    }
    
    public void onSoundPreferences(Displayable d)
    {
        m_objDisplayableStack.push(d);
        m_objSoundDialogue.readSettings();
        m_objDisplay.setCurrent(m_objSoundDialogue);
    }
    
    public void pauseApp()
    {
        m_objGame.pause();
    }
    
    public void repaintView()
    {
        m_objCanvas.repaint();
    }
    
    public void restartGame()
    {
        m_objCanvas.onNewGame();
        m_objGame.restart();
        m_objDisplay.setCurrent(m_objCanvas);
    }
    
    public void resumeGame()
    {
        m_objGame.resume();
        m_objDisplay.setCurrent(m_objCanvas);
    }
    
    public void setColor(int nParam, int nColor)
    {
        m_objSettingsKeeper.setBricksColor(nColor);
        m_objCanvas.setBricksColor(nColor);
    }
    
    public void setLocale(int nLocale)
    {
        m_objLocalizer.setLocale(nLocale);
    }
    
    public void setMute(boolean b)
    {
        m_objGame.getSounder().setMute(b);
    }
    
    public void setVolume(int n)
    {
        m_objGame.getSounder().setVolume(n);
    }
    
    public void startGame()
    {
        m_objCanvas.onNewGame();
        m_objGame.start();
        m_objDisplay.setCurrent(m_objCanvas);
    }
    
    public void startApp()
    {
        m_objGame.resume();
    }
}